#include "3dload.h"

//BMP RGB 24 bpp uniquement...
int ImageLoad(char *filename, Image *image) {
    FILE *file;
    unsigned long size;                 // size of the image in bytes.
    unsigned long i;                    // standard counter.
    unsigned short int planes;          // number of planes in image (must be 1) 
    unsigned short int bpp;             // number of bits per pixel (must be 24)
    char temp;                          // temporary color storage for bgr-rgb conversion.

    // make sure the file is there.
    if ((file = fopen(filename, "rb"))==NULL)
    {
        printf("File Not Found : %s\n",filename);
        return 0;
    }
    
    // seek through the bmp header, up to the width/height:
    fseek(file, 18, SEEK_CUR);

    // read the width
    if ((i = fread(&image->sizeX, 4, 1, file)) != 1) {
        printf("Error reading width from %s.\n", filename);
        return 0;
    }
    //printf("Width of %s: %lu\n", filename, image->sizeX);
    
    // read the height 
    if ((i = fread(&image->sizeY, 4, 1, file)) != 1) {
        printf("Error reading height from %s.\n", filename);
        return 0;
    }
    //printf("Height of %s: %lu\n", filename, image->sizeY);
    
    // read the planes
    if ((fread(&planes, 2, 1, file)) != 1) {
        printf("Error reading planes from %s.\n", filename);
        return 0;
    }
    if (planes != 1) {
        printf("Planes from %s is not 1: %u\n", filename, planes);
        return 0;
    }

    // read the bpp
    if ((i = fread(&bpp, 2, 1, file)) != 1) {
        printf("Error reading bpp from %s.\n", filename);
        return 0;
    }
    if ((bpp != 24) && (bpp!=32)) {
        printf("Bpp from %s is not 24 or 32: %u\n", filename, bpp);
        return 0;
    }
    
    // calculate the size (assuming 24 bits or 3 bytes per pixel).
    size = image->sizeX * image->sizeY * (bpp/8);

    // seek past the rest of the bitmap header.
    fseek(file, 24, SEEK_CUR);

    // read the data. 
    image->data = (char *) malloc(size);
    if (image->data == NULL) {
        printf("Error allocating memory for color-corrected image data");
        return 0;       
    }

    if ((i = fread(image->data, size, 1, file)) != 1) {
        printf("Error reading image data from %s.\n", filename);
        return 0;
    }

    for (i=0;i<size;i+=(bpp/8)) { // reverse all of the colors. (bgr -> rgb)
        temp = image->data[i];
        image->data[i] = image->data[i+2];
        image->data[i+2] = temp;
    }
    // we're done.
    return bpp;
}

void LoadTexture(char*filename)
 {
  Image im;
  int bpp;
  
  if(bpp=ImageLoad(filename,&im))
   {
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	if(bpp==24)
	glTexImage2D(GL_TEXTURE_2D,0,3,
                im.sizeX,im.sizeY,
                0,GL_RGB,GL_UNSIGNED_BYTE,im.data);
	else if(bpp==32)
    glTexImage2D(GL_TEXTURE_2D,0,4,
                im.sizeX,im.sizeY,
                0,GL_RGBA,GL_UNSIGNED_BYTE,im.data);
   }
 };

void MyMaterial(GLenum mode,GLfloat *f,GLfloat alpha)
{
 GLfloat d[4];
 d[0]=f[0];
 d[1]=f[1];
 d[2]=f[2];
 d[3]=alpha;
 glMaterialfv (GL_FRONT_AND_BACK,mode,d);
}

/*
 *  SelectMaterial uses OpenGL commands to define facet colors.
 *
 *  Returns:
 *    Nothing
 */

void SelectMaterial(int i,struct sample_TEXTURE * texture_maps,struct sample_MATERIAL * materials)
{
  //
  // Define the reflective properties of the 3D Object faces.
  //
  GLfloat alpha;
  
  glEnd();
  alpha=materials[i].alpha;
  MyMaterial (GL_AMBIENT, materials[i].ambient,alpha);
  MyMaterial (GL_DIFFUSE, materials[i].diffuse,alpha);
  MyMaterial (GL_SPECULAR, materials[i].specular,alpha);
  MyMaterial (GL_EMISSION, materials[i].emission,alpha);
  glMaterialf (GL_FRONT_AND_BACK,GL_SHININESS,materials[i].phExp);
	glEnd();
	if(materials[i].texture>-1)
	{
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D,texture_maps[materials[i].texture].id);
	}
	else
	glDisable(GL_TEXTURE_2D);
	glBegin(GL_TRIANGLES);

  glBegin(GL_TRIANGLES);
};

GLint Gen3DModel(MODEL m)
{
 int i;
 int j;
 int mcount,mindex;
 char chaine[120];
 GLint lid;
 
 
for(i=0;i<m.nb_texture_maps;i++)
  {
    GLuint texture_name;
    glGenTextures(1,&texture_name);
    m.texture_maps[i].id=texture_name;
    glBindTexture(GL_TEXTURE_2D,texture_name);
	sprintf(chaine,"%s%s/%s",TEXTURE_PATH,m.nom,m.texture_maps[i].name);
    LoadTexture(chaine);
  }

 lid=glGenLists(1);
	mcount=0;
	mindex=0;
   glNewList(lid, GL_COMPILE);

    glBegin (GL_TRIANGLES);
      for(i=0;i<m.nb_face_indicies;i++)
       {
      if(!mcount)
       {
        SelectMaterial(m.material_ref[mindex].mr[0],m.texture_maps,m.materials);
        mcount=m.material_ref[mindex].mr[1];
        mindex++;
       }
       mcount--;
       for(j=0;j<3;j++)
        {
          int vi=m.face_indicies[i].x[j];
          int ni=m.face_indicies[i].y[j];//Normal index
          int ti=m.face_indicies[i].z[j];//Texture index
           glNormal3f (m.normals[ni].n[0],m.normals[ni].n[1],m.normals[ni].n[2]);
           glTexCoord2f(m.textures[ti].t[0],m.textures[ti].t[1]);
           glVertex3f (m.vertices[vi].v[0],m.vertices[vi].v[1],m.vertices[vi].v[2]);
        }
       }
    glEnd ();

   glEndList();
   return lid;
};


MODEL load_model(char * file)
{
	FILE * f;
	int i=0;
	short tex_len=0;
	MODEL m;

	f = fopen(file,"rb+");
	fread(m.nom,sizeof(char)*20,1,f);
	fread(&(m.nb_materials),sizeof(long),1,f);
	fread(&(m.nb_texture_maps),sizeof(long),1,f);
	fread(&(m.nb_face_indicies),sizeof(long),1,f);
	fread(&(m.nb_vertices),sizeof(long),1,f);
	fread(&(m.nb_normals),sizeof(long),1,f);
	fread(&(m.nb_textures),sizeof(long),1,f);
	fread(&(m.nb_material_ref),sizeof(long),1,f);
	m.materials = (struct sample_MATERIAL *)malloc(sizeof(struct sample_MATERIAL)*m.nb_materials);
	fread((m.materials),sizeof(struct sample_MATERIAL),m.nb_materials,f);
	m.texture_maps = (struct sample_TEXTURE *)malloc(sizeof(struct sample_TEXTURE)*m.nb_texture_maps);
	fread(m.texture_maps,sizeof(struct sample_TEXTURE),m.nb_texture_maps,f);
	m.face_indicies = (T_TRIANGLE *)malloc(sizeof(T_TRIANGLE)*m.nb_face_indicies);
	fread(m.face_indicies,sizeof(T_TRIANGLE),m.nb_face_indicies,f);
	m.vertices = (T_VERTEX *)malloc(sizeof(T_VERTEX)*m.nb_vertices);
	fread(m.vertices,sizeof(T_VERTEX),m.nb_vertices,f);
	m.normals = (T_NORMS *)malloc(sizeof(T_NORMS)*m.nb_normals);
	fread(m.normals,sizeof(T_NORMS),m.nb_normals,f);
	m.textures = (T_TEX *)malloc(sizeof(T_TEX)*m.nb_textures);
	fread(m.textures,sizeof(T_TEX),m.nb_textures,f);
	m.material_ref = (T_MAT_REF *)malloc(sizeof(T_MAT_REF)*m.nb_material_ref);
	fread(m.material_ref,sizeof(T_MAT_REF),m.nb_material_ref,f);
	fclose(f);
	return m;
}

/*
int write_model(char * file)
{
	FILE * f;
	int i=0;
	short tex_len=0;
	char nom[21];

	f = fopen(file,"wb+");

	//File Header
	long nb_face_indicies = sizeof(face_indicies)/sizeof(face_indicies[0]);
	long nb_vertices = sizeof(vertices)/sizeof(vertices[0]);
	long nb_normals = sizeof(normals)/sizeof(normals[0]);
	long nb_textures = sizeof(textures)/sizeof(textures[0]);
	long nb_material_ref = sizeof(material_ref)/sizeof(material_ref[0]);
	strcpy(nom,NOM_MODELE);
	for(i=strlen(NOM_MODELE)+1;i<21;i++)
		nom[i]=0;

	fwrite(nom,sizeof(char)*20,1,f);
	fwrite(&nb_materials,sizeof(long),1,f);
	fwrite(&nb_texture_maps,sizeof(long),1,f);
	fwrite(&nb_face_indicies,sizeof(long),1,f);
	fwrite(&nb_vertices,sizeof(long),1,f);
	fwrite(&nb_normals,sizeof(long),1,f);
	fwrite(&nb_textures,sizeof(long),1,f);
	fwrite(&nb_material_ref,sizeof(long),1,f);
	fwrite(materials,sizeof(sample_MATERIAL),nb_materials,f);
	fwrite(texture_maps,sizeof(sample_TEXTURE),nb_texture_maps,f);
	fwrite(face_indicies,9*sizeof(short),nb_face_indicies,f);
	fwrite(vertices,3*sizeof(GLfloat),nb_vertices,f);
	fwrite(normals,3*sizeof(GLfloat),nb_normals,f);
	fwrite(textures,2*sizeof(GLfloat),nb_textures,f);
	fwrite(material_ref,2*sizeof(int),nb_material_ref,f);

	fclose(f);
	return 1;
}*/


