/* Nom : 3dload.h
   Auteur : Laurent-Mickael Ellena
   Description : Module de chargement de modeles 3D stocks selon
   un format propritaire simpliste. Les structures de donnes sont
   disponibles pour les courageux souhaitant faire des modifications.
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>       /* for cos(), sin(), and sqrt() */
#include <GL/glut.h>

/* PARAMETRE INDIQUANT LE REPERTOIRE DES TEXTURES */
#define TEXTURE_PATH "textures/"

/* Image type - contains height, width, and data */
struct Image {
    unsigned long sizeX;
    unsigned long sizeY;
    char *data;
};
typedef struct Image Image;

typedef struct point {
	float x,y,z;
} T_POINT;

typedef struct vecteur {
	float x,y,z;
} T_VECTEUR;

struct sample_MATERIAL{
 GLfloat ambient[3];
 GLfloat diffuse[3];
 GLfloat specular[3];
 GLfloat emission[3];
 GLfloat alpha;
 GLfloat phExp;
 int   texture;
};
struct sample_TEXTURE{
 char name[21];
 GLint  id;
};

typedef struct t_triangle{
	short x[3];
	short y[3];
	short z[3];
} T_TRIANGLE ;

typedef struct t_vertex{
	GLfloat v[3];
} T_VERTEX;

typedef struct t_norms{
	GLfloat n[3];
} T_NORMS;

typedef struct t_tex{
	GLfloat t[2];
} T_TEX;

typedef struct t_mat_ref{
	int mr[2];
} T_MAT_REF;

typedef struct model {
	char nom[20];
	long nb_materials;
	long nb_texture_maps;
	long nb_face_indicies;
	long nb_vertices;
	long nb_normals;
	long nb_textures;
	long nb_material_ref;
	struct sample_MATERIAL * materials;
	struct sample_TEXTURE * texture_maps;
	T_TRIANGLE * face_indicies;
	T_VERTEX * vertices;
	T_NORMS * normals;
	T_TEX * textures;
	T_MAT_REF * material_ref;
} MODEL;

//BMP RGB ou RGBA (24 ou 32 bits)
int ImageLoad(char *filename, Image *image);

/* Chargement d'une texture sous OpenGL */
/* l'image source de la texture doit etre une 
 BMP au format RGB(24bits) ou RGBA (32 bits) */
void LoadTexture(char*filename);

/* Fonction de chargement des donnes du modele */
MODEL load_model(char * file);

/* Generateur de Display List */
GLint Gen3DModel(MODEL m);